﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;
using PivotExpandIssue.LBG.Pricing.LiveRiskModule.Model.DataItems;

namespace PivotExpandIssue
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private FlatDataSource _dataSource;
        protected const string NumberFormat = "{0:#,###}";

        protected const string DateFormat = "dd-MMM-yyyy";

        public MainWindow()
        {
            InitializeComponent();

            _dataSource = new FlatDataSource();
            IEnumerable<DimensionMetadata> dimensionMetadata = BuildDimensionMetadata().ToArray();
            DimensionMetadata = dimensionMetadata.ToDictionary(dm => dm.DisplayName, dm => dm);
            CubeMetadata = BuildCubeMetadata();
            _dataSource.CubesSettings.Add(CubeMetadata);
            _dataSource.PreserveMembersOrder = false;
            _dataSource.LevelFirstSortDirection = ListSortDirection.Ascending;
            _dataSource.DimensionsGenerationMode = DimensionsGenerationMode.Metadata;
            
            var dataItems = new ObservableCollection<DataItem>(BuildData());
            _dataSource.ItemsSource = dataItems;
            _dataSource.Initialized += MyFlatDataSource_OnInitialized;

            slicer.SlicerProvider = _dataSource;
            pivotGrid.DataSource = _dataSource;
            dataSelector.DataSource = _dataSource;
            Loaded += MainWindow_Loaded;
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            pivotGrid.AllowSuperCompactLayout = true;
            //pivotGrid.AllowCompactLayout = true;
        }

        private void MyFlatDataSource_OnInitialized(object sender, EventArgs e)
        {
            _dataSource.LoadCustomizations(Constants.Customizations);
        }

        private IEnumerable<DataItem> BuildData()
        {
            for (int a = 0; a < 2; a++)
            {
                string book = "book" + a;
                for (int bucketNameNum = 0; bucketNameNum < 2; bucketNameNum++)
                {
                    string bucketName = "bucketName" + bucketNameNum;
                    for (int businessNameNum = 0; businessNameNum < 2; businessNameNum++)
                    {
                        string businessName = "businessName" + businessNameNum;
                        for (int categoryNum = 0; categoryNum < 2; categoryNum++)
                        {
                            string category = "category" + categoryNum;
                            for (int categoryGroupNum = 0; categoryGroupNum < 2; categoryGroupNum++)
                            {
                                string categoryGroup = "categoryGroup" + categoryGroupNum;
                                for (int ccyNum = 0; ccyNum < 2; ccyNum++)
                                {
                                    string ccy = "ccy" + ccyNum;
                                    for (int curveBucketNum = 0; curveBucketNum < 2; curveBucketNum++)
                                    {
                                        string curveBucket = "curveBucket" + curveBucketNum;
                                        for (int factorNum= 0; factorNum < 2; factorNum++)
                                        {
                                            string factor = "factor" + factorNum;
                                            for (int maturityNum = 0; maturityNum < 2; maturityNum++)
                                            {
                                                DateTime maturity = DateTime.Now.AddYears(maturityNum);
                                                for (int podShortNum = 0; podShortNum < 2; podShortNum++)
                                                {
                                                    string podShort = "podShort" + podShortNum;
                                                    for (int productNum = 0; productNum < 2; productNum++)
                                                    {
                                                        string product = "product" + productNum;
                                                        for (int productDescriptionNum = 0; productDescriptionNum < 2; productDescriptionNum++)
                                                        {
                                                            string productDescription = "productDescription" + productDescriptionNum;
                                                            for (int QlProductNum = 0; QlProductNum < 2; QlProductNum++)
                                                            {
                                                                string qlProduct = "qlProduct" + QlProductNum;
                                                                for (int RiskUnderlyingNameNum = 0; RiskUnderlyingNameNum < 2; RiskUnderlyingNameNum++)
                                                                {
                                                                    string riskUnderlyingName = "riskUnderlyingName" + RiskUnderlyingNameNum;
                                                                    for (int tradeTypeNum = 0; tradeTypeNum < 2; tradeTypeNum++)
                                                                    {
                                                                        string tradeType = "tradeType" + tradeTypeNum;
                                                                        for (int traderNum = 0; traderNum < 2; traderNum++)
                                                                        {
                                                                            string trader = "trader" + traderNum;


                                                                            var dataItem = new DataItem()
                                                                            {
                                                                                Book = book,
                                                                                BucketName = bucketName,
                                                                                Business = businessName,
                                                                                Category = category,
                                                                                CategoryGroup = categoryGroup,
                                                                                Ccy = ccy,
                                                                                CurveBucket = curveBucket,
                                                                                Factor = factor,
                                                                                Maturity = maturity,
                                                                                PodShort = podShort,
                                                                                Product = product,
                                                                                ProductDescription = productDescription,
                                                                                QlProduct = qlProduct,
                                                                                RiskUnderlyingName = riskUnderlyingName,
                                                                                TradeType = tradeType,
                                                                                Trader = trader,
                                                                                Value = 10,
                                                                                ValueGbp = 10
                                                                            };
                                                                            yield return dataItem;
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }

                                }

                            }

                        }
                        
                    }
                }


            

            }
        }


        protected IEnumerable<DimensionMetadata> BuildDimensionMetadata()
        {
            var businessDimension
               = new DimensionMetadata() { SourcePropertyName = "Business", DisplayName = "Business"};
            AddDefaultHierarchyToDimension<DataItem>(businessDimension, p => p.Business, "Business", "All Businesses");
            yield return businessDimension;

            var podDimension
                = new DimensionMetadata() { SourcePropertyName = "PodShort", DisplayName = "Pod" };
            AddDefaultHierarchyToDimension<DataItem>(podDimension, p => p.PodShort, "Pod", "All Pods");
            yield return podDimension;

            var bookDimension
                = new DimensionMetadata() { SourcePropertyName = "Book", DisplayName = "Book"};
            AddDefaultHierarchyToDimension<DataItem>(bookDimension, p => p.Book, "Book", "All Books");
            yield return bookDimension;

            var traderDimension
 = new DimensionMetadata() { SourcePropertyName = "Trader", DisplayName = "Trader"};
            AddDefaultHierarchyToDimension<DataItem>(traderDimension, p => p.Trader, "Trader", "All Traders");
            yield return traderDimension;

            var ccyDimension
                = new DimensionMetadata() { SourcePropertyName = "Ccy", DisplayName = "Ccy"};
            AddDefaultHierarchyToDimension<DataItem>(ccyDimension, p => p.Ccy, "Ccy");
            yield return ccyDimension;

            var categoryDimension
                = new DimensionMetadata() { SourcePropertyName = "Category", DisplayName = "Category"};
            AddDefaultHierarchyToDimension<DataItem>(categoryDimension, p => p.Category, "Category", "All Categories");
            yield return categoryDimension;

            DimensionMetadata riskUnderlyingNameDimension
    = new DimensionMetadata() { SourcePropertyName = "RiskUnderlyingName", DisplayName = "Risk Underlying Name"};
            AddDefaultHierarchyToDimension<DataItem>(riskUnderlyingNameDimension, p => p.RiskUnderlyingName,
                "Risk Underlying Name", "All Risk Underlying Names");
            yield return riskUnderlyingNameDimension;

            var categoryGroupDimension
                = new DimensionMetadata() { SourcePropertyName = "CategoryGroup", DisplayName = "CategoryGroup"};
            AddDefaultHierarchyToDimension<DataItem>(categoryGroupDimension, p => p.CategoryGroup, "CategoryGroup", "All Category Groups");
            yield return categoryGroupDimension;

            var curveBucketDimension
            = new DimensionMetadata() { SourcePropertyName = "CurveBucket", DisplayName = "CurveBucket" };
            AddDefaultHierarchyToDimension<DataItem>(curveBucketDimension, p => p.CurveBucket, "CurveBucket", "All Curve Buckets");
            yield return curveBucketDimension;

            var maturityDimension
                = new DimensionMetadata() { SourcePropertyName = "Maturity", DisplayName = "Maturity" };
            maturityDimension.DisplayFormat = DateFormat;
            AddDefaultHierarchyToDimension<DataItem>(maturityDimension, p => p.Maturity, "Maturity", "All Maturities");
            yield return maturityDimension;

            var bucketDimension
                = new DimensionMetadata() { SourcePropertyName = "BucketName", DisplayName = "Bucket" };
            AddDefaultHierarchyToDimension<DataItem>(bucketDimension, p => p.BucketName, "Bucket", "All Buckets");
            yield return bucketDimension;

            var qlProductDimension
                = new DimensionMetadata() { SourcePropertyName = "QlProduct", DisplayName = "QL Product"};
            AddDefaultHierarchyToDimension<DataItem>(qlProductDimension, p => p.QlProduct, "QlProduct", "All QL Products");
            yield return qlProductDimension;

            DimensionMetadata tradeTypeDimension
                = new DimensionMetadata() { SourcePropertyName = "TradeType", DisplayName = "Trade Type"};
            AddDefaultHierarchyToDimension<DataItem>(tradeTypeDimension, p => p.TradeType, "Trade Type", "All Trade Types");
            yield return tradeTypeDimension;

            var productDimension
               = new DimensionMetadata() { SourcePropertyName = "Product", DisplayName = "Product"};
            AddDefaultHierarchyToDimension<DataItem>(productDimension, p => p.Product, "Product", "All Products");
            yield return productDimension;

            var measureMetadata = new DimensionMetadata
            {
                DimensionType = DimensionType.Measure,
                SourcePropertyName = "Value",
                DisplayName = "Value",
                DisplayFormat = NumberFormat,
                AggregatorType = AggregatorType.Sum
            };

            yield return measureMetadata;

            var gbpMeasureMetadata = new DimensionMetadata
            {
                DimensionType = DimensionType.Measure,
                SourcePropertyName = "ValueGbp",
                DisplayName = "ValueGBP",
                DisplayFormat = NumberFormat,
                AggregatorType = AggregatorType.Sum
            };

            yield return gbpMeasureMetadata;
        }        


        private CubeMetadata BuildCubeMetadata()
        {
            CubeMetadata cubeMetadata = new CubeMetadata();
            cubeMetadata.DisplayName = "Grouped CR";
            cubeMetadata.DataTypeFullName = typeof(DataItem).FullName;
            cubeMetadata.DimensionSettings.AddRange(DimensionMetadata.Values);
            return cubeMetadata;
        }

        public IDictionary<string, DimensionMetadata> DimensionMetadata { get; set; }
        public CubeMetadata CubeMetadata { get; private set; }
       
        
        protected void AddDefaultHierarchyToDimension<TDataItem>(DimensionMetadata dimension,
            Expression<Func<TDataItem, object>> propertyExpression, string hierarchyName, string allName=null)
        {
            HierarchyDescriptor<TDataItem> hierarchy = new HierarchyDescriptor<TDataItem>(propertyExpression)
            {
                HierarchyDisplayName = hierarchyName
            };

            if(!string.IsNullOrEmpty(allName))
            {
                hierarchy.AddLevel(p => allName, allName);
            }
            
            hierarchy.AddLevel(propertyExpression, hierarchyName);
            dimension.HierarchyDescriptors.Add(hierarchy);
        }
    }
}
